﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Data.Services.Client;
using System.Diagnostics;
using System.Linq;
using Adressverwaltung.Data.Contexts;
using Adressverwaltung.Data.Models;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace Adressverwaltung.Data.Repositories
{
    [Export(typeof(IAdressverwaltungRepository))]
    public class AdressverwaltungTableStorageRepository : IAdressverwaltungRepository
    {
        private static readonly CloudStorageAccount StorageAccount;
        private readonly AdressverwaltungTableStorageContext _context;

        #region Konstruktor

        static AdressverwaltungTableStorageRepository()
        {
            StorageAccount = CloudStorageAccount.FromConfigurationSetting("AzureStorageConnectionString");

            Trace.TraceInformation("Erzeuge Table Storage Elemente anhand des Datenmodels.");
            CloudTableClient.CreateTablesFromModel(
                typeof(AdressverwaltungTableStorageContext),
                StorageAccount.TableEndpoint.AbsoluteUri,
                StorageAccount.Credentials);
        }

        public AdressverwaltungTableStorageRepository()
        {
            _context = new AdressverwaltungTableStorageContext(
                StorageAccount.TableEndpoint.AbsoluteUri
                , StorageAccount.Credentials)
            {
                RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromSeconds(1))
            };
        }

        #endregion

        #region Adressbuch Methoden

        public IEnumerable<Adressbuch> GetAdressbuecher()
        {
            Trace.TraceInformation("Frage alle Adressbücher des Benutzers ab.");
            return _context.Adressbuecher
                .Where(k => k.PartitionKey == "UserName")
                .AsTableServiceQuery()
                .Execute()
                .OrderBy(b => b.Name);
        }

        public Adressbuch LoadAdressbuch(Guid adressbuchId)
        {
            Trace.TraceInformation("Lade Adressbuch: Id = {0}.", adressbuchId);
            var adressbuecher = from b in _context.Adressbuecher
                                where b.PartitionKey == "UserName"
                                && b.RowKey == adressbuchId.ToString()
                                select b;
            return adressbuecher.FirstOrDefault();
        }

        public void AddAdressbuch(Adressbuch adressbuch)
        {
            Trace.TraceInformation("Füge neues Adressbuch hinzu: Id = {0}", adressbuch.AdressbuchId);
            _context.AddObject("Adressbuecher", adressbuch);
            _context.SaveChanges();
        }

        public void UpdateAdressbuch(Guid adressbuchId, Adressbuch adressbuch)
        {
            var aktuellesAdressbuch = LoadAdressbuch(adressbuchId);

            if (aktuellesAdressbuch.Name == adressbuch.Name) return;

            Trace.TraceInformation("Aktualisiere Adressbuch: Id = {0}, Alter Name = '{1}', Neuer Name = '{2}'.", adressbuchId, aktuellesAdressbuch.Name, adressbuch.Name);
            aktuellesAdressbuch.Name = adressbuch.Name;
            _context.UpdateObject(aktuellesAdressbuch);
            _context.SaveChanges();
        }

        public void DeleteAdressbuch(Guid adressbuchId)
        {
            Trace.TraceInformation("Lösche alle Kontakte im Adressbuch: Id = {0}.", adressbuchId);
            // Lösche alle Kontakte im Adressbuch
            var loeschZaehler = 0;
            foreach (var kontakt in _context.Kontakte
                .Where(k => k.PartitionKey == adressbuchId.ToString())
                .AsTableServiceQuery()
                .Execute())
            {
                _context.DeleteObject(kontakt);
                loeschZaehler++;

                if (loeschZaehler == 100)
                {
                    // Alle Löschoperationen werden innerhalb 
                    // einer Transaktion ausgeführt. (Max. 100 Operationen)
                    _context.SaveChanges(SaveChangesOptions.Batch);
                    loeschZaehler = 0;
                    Trace.TraceInformation("100 Kontakte im Adressbuch: Id = {0} gelöscht.", adressbuchId);
                }
            }
            // Ausführen der noch offenen Löschoperationen.
            if (loeschZaehler > 0)
            {
                _context.SaveChanges(SaveChangesOptions.Batch);
                Trace.TraceInformation("{0} Kontakte im Adressbuch: Id = {1} gelöscht.", loeschZaehler, adressbuchId);
            }

            Trace.TraceInformation("Lösche Adressbuch: Id = {0}.", adressbuchId);
            // Lösche Adressbuch
            var adressbuch = LoadAdressbuch(adressbuchId);
            _context.DeleteObject(adressbuch);
            _context.SaveChanges();
        }

        #endregion

        #region Kontakt Methoden

        public IEnumerable<Kontakt> GetKontakte(Guid adressbuchId)
        {
            Trace.TraceInformation("Frage alle Kontakte des Adressbuchs (Id = {0}) ab.", adressbuchId);
            return _context.Kontakte
                .Where(k => k.PartitionKey == adressbuchId.ToString())
                .AsTableServiceQuery()
                .Execute()
                .OrderBy(k => k.Nachname)
                .ThenBy(k => k.Vorname);
        }

        public Kontakt LoadKontakt(Guid adressbuchId, Guid kontaktId)
        {
            Trace.TraceInformation("Lade Kontakt: Id = {0}.", kontaktId);
            var kontakte = from k in _context.Kontakte
                           where k.PartitionKey == adressbuchId.ToString()
                           && k.RowKey == kontaktId.ToString()
                           select k;
            return kontakte.FirstOrDefault();
        }

        public void AddKontakt(Guid adressbuchId, Kontakt kontakt)
        {
            Trace.TraceInformation("Füge neuen Kontakt (Id = {0}) dem Adressbuch (Id = {1}) hinzu.", kontakt.RowKey, adressbuchId);
            kontakt.PartitionKey = adressbuchId.ToString();

            _context.AddObject("Kontakte", kontakt);
            _context.SaveChanges();
        }

        public void UpdateKontakt(Guid adressbuchId, Guid kontaktId, Kontakt kontakt)
        {
            var aktuellerKontakt = LoadKontakt(adressbuchId, kontaktId);

            if (aktuellerKontakt.Vorname == kontakt.Vorname
            && aktuellerKontakt.Nachname == kontakt.Nachname
            && aktuellerKontakt.Firma == kontakt.Firma
            && aktuellerKontakt.Strasse == kontakt.Strasse
            && aktuellerKontakt.PLZ == kontakt.PLZ
            && aktuellerKontakt.Ort == kontakt.Ort
            && aktuellerKontakt.Telefon == kontakt.Telefon
            && aktuellerKontakt.Mobil == kontakt.Mobil
            && aktuellerKontakt.Webseite == kontakt.Webseite) return;

            aktuellerKontakt.Vorname = kontakt.Vorname;
            aktuellerKontakt.Nachname = kontakt.Nachname;
            aktuellerKontakt.Firma = kontakt.Firma;
            aktuellerKontakt.Strasse = kontakt.Strasse;
            aktuellerKontakt.PLZ = kontakt.PLZ;
            aktuellerKontakt.Ort = kontakt.Ort;
            aktuellerKontakt.Telefon = kontakt.Telefon;
            aktuellerKontakt.Mobil = kontakt.Mobil;
            aktuellerKontakt.Webseite = kontakt.Webseite;

            Trace.TraceInformation("Aktualisiere Kontakt: Id = {0}.", aktuellerKontakt.KontaktId);
            _context.UpdateObject(aktuellerKontakt);
            _context.SaveChanges();
        }

        public void DeleteKontakt(Guid adressbuchId, Guid kontaktId)
        {
            Trace.TraceInformation("Lösche Kontakt: Id = {0}.", kontaktId);
            var kontakt = LoadKontakt(adressbuchId, kontaktId);

            _context.DeleteObject(kontakt);
            _context.SaveChanges();
        }

        public void UpdateKontaktFotoUrl(Guid adressbuchId, Guid kontaktId, string fotoUrl)
        {
            var aktuellerKontakt = LoadKontakt(adressbuchId, kontaktId);

            if (aktuellerKontakt.FotoUrl == fotoUrl) return;
            aktuellerKontakt.FotoUrl = fotoUrl;

            Trace.TraceInformation("Aktualisiere die Foto URL des Kontakts: Id = {0}.", kontaktId);
            _context.UpdateObject(aktuellerKontakt);
            _context.SaveChanges();
        }

        #endregion

    }
}
