﻿using System;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Net;
using System.Reflection;
using Adressverwaltung.Data.Models;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace Adressverwaltung.Data.Repositories
{
    [Export(typeof(IFotoRepository))]
    public class BlobFotoRepository : IFotoRepository
    {
        private const string BlobNameBilder = "bilder";
        private const int MaxBildGroesse = 80;

        [Import(typeof(IAdressverwaltungRepository))]
        private IAdressverwaltungRepository _repository;

        private static bool _initialisiert;
        private static readonly object Sperre = new Object();
        private static CloudBlobClient _blobStorage;

        static BlobFotoRepository()
        {
            Speicherplatzinitialisierung();
        }

        public BlobFotoRepository()
        {
            using (var catalog = new AssemblyCatalog(Assembly.GetExecutingAssembly()))
            using (var container = new CompositionContainer(catalog))
                container.ComposeParts(this);
        }

        static void Speicherplatzinitialisierung()
        {
            if (_initialisiert)
            {
                return;
            }
            lock (Sperre)
            {
                if (_initialisiert)
                {
                    return;
                }
                try
                {
					// Die Zugangsdaten werden hierzu
					// aus der Rolleneinstellung 
					// "AzureStorageConnectionString" gelesen.
                    var storageAccount = CloudStorageAccount.FromConfigurationSetting("AzureStorageConnectionString");
                    _blobStorage = storageAccount.CreateCloudBlobClient();

                    // Öffentlichen Speicherplatz erstellen
                    var containerOeffentlich = _blobStorage.GetContainerReference(BlobNameBilder);
                    containerOeffentlich.CreateIfNotExist();
                    Trace.TraceInformation("Blob Storage für die Bilder erzeugt.");

                    // Rechte für den öffentlichen Speicherplatz vergeben
                    var permissions = containerOeffentlich.GetPermissions();
                    permissions.PublicAccess = BlobContainerPublicAccessType.Container;
                    containerOeffentlich.SetPermissions(permissions);
                    Trace.TraceInformation("Rechte für den Blob Storage gesetzt.");
                }
                catch (WebException)
                {
                    throw new WebException(
                        "Speicherplatzinitialisierung fehlgeschlagen. "
                        + "Bitte überprüfen Sie die Konfiguration.");
                }
                _initialisiert = true;
            }
        }

        public void KontaktBildHinzufuegen(Stream bild, string dateiName, string contentType, Kontakt kontakt)
        {
            // Bild hochladen
            var bildBlobName = string.Format("{0}/{1}{2}",
                BlobNameBilder,
                kontakt.KontaktId,
                Path.GetExtension(dateiName));
            var blob = _blobStorage.GetBlockBlobReference(bildBlobName);

            using (var output = blob.OpenWrite())
            {
                VerkleinereBild(bild, output);

                // Blob Speichervorgang abschließen und Eigenschaften setzen
                output.Commit();
                blob.Properties.ContentType = "image/jpeg";
                blob.SetProperties();

                // Kontakt Bild-URL speichern.
                _repository.UpdateKontaktFotoUrl(
                    kontakt.AdressbuchId, 
                    kontakt.KontaktId, 
                    blob.Uri.ToString());
                Trace.TraceInformation("Bild '{0}' im Blob Storage gespeichert als '{1}'", dateiName, bildBlobName);
            }
        }

        public void KontaktBildEntfernen(Guid kontaktId)
        {
            // Lösche die Miniaturansicht des Kontakts
            BildEntfernen(kontaktId);
        }

        public void AdressbuchBilderEntfernen(Guid adressbuchId)
        {
            // Lösche alle Miniaturansichten im Adressbuch
            Trace.TraceInformation("Lösche alle Kontakte im Adressbuch: Id = {0}.", adressbuchId);
            foreach (var kontakt in _repository.GetKontakte(adressbuchId))
                BildEntfernen(kontakt.KontaktId);
        }

        private static void BildEntfernen(Guid kontaktId)
        {
            var miniaturansichtBlobName = string.Format("{0}/{1}.jpg",
                BlobNameBilder,
                kontaktId);
            var blob = _blobStorage.GetBlockBlobReference(miniaturansichtBlobName);
            blob.DeleteIfExists();
            Trace.TraceInformation("Bild gelöscht: '{0}'.", miniaturansichtBlobName);
        }

        private static void VerkleinereBild(Stream input, Stream output)
        {
            if (input == null) throw new ArgumentNullException("input");
            if (output == null) throw new ArgumentNullException("output");

            // Neue Höhe und Breite berechnen
            var originalBild = new Bitmap(input);
            var breite = originalBild.Width > originalBild.Height
                ? MaxBildGroesse
                : (MaxBildGroesse * originalBild.Width / originalBild.Height);
            var hoehe = originalBild.Width > originalBild.Height
                ? (MaxBildGroesse * originalBild.Height / originalBild.Width)
                : MaxBildGroesse;

            // Bild verkleinern
            var miniaturBild = new Bitmap(breite, hoehe);
            using (var graphics = Graphics.FromImage(miniaturBild))
            {
                graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;
                graphics.SmoothingMode = SmoothingMode.AntiAlias;
                graphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
                graphics.DrawImage(originalBild, 0, 0, breite, hoehe);
            }

            // Bild speichern
            miniaturBild.Save(output, ImageFormat.Jpeg);
        }
    }
}